// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020-2022 Florian Schmaus
#include "Fiber.hpp"

#include <iostream>	 // for operator<<, basic_ostream, ostream, basic_ostrea...
#include <typeinfo>	 // for type_info

#include "emper-config.h"	 // IWYU pragma: keep

void Fiber::run() {
	LOGD("run() calling " << function.target<FIBER_FUN_TEMPLATE_ARG>() << " ("
												<< function.target_type().name() << ") with arg " << arg);
	function(arg);
}

auto Fiber::isRunnable() const -> bool { return runnable.load(std::memory_order_relaxed); }

void Fiber::printTo(std::ostream &strm, bool withPtr) const {
	strm << "Fiber [";
	if (withPtr) {
		strm << "ptr=" << this << " ";
	}
	// clang-format: off
	strm << "func=" << (function.target<void(void *)>() != nullptr) << "arg=" << arg;
	// clang-format: on

	if (affinity) {
		strm << " aff=" << affinity;
	} else {
		strm << " aff=nullptr";
	}

	strm << "]";
}
