// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2021 Florian Fischer
#pragma once

#include <stdexcept>

#include "CallerEnvironment.hpp"
#include "Emper.hpp"
#include "Worker.hpp"

namespace emper::sleep_strategy {

template <class T>
class AbstractWorkerSleepStrategy {
	[[nodiscard]] inline auto getSleeping() const -> long {
		return static_cast<T*>(this)->getSleeping();
	}

	template <CallerEnvironment callerEnvironment>
	inline void notifyMany(unsigned count) {
		static_cast<T*>(this)->template notifyMany<callerEnvironment>(count);
	}

	template <CallerEnvironment callerEnvironment>
	inline void notifyOne() {
		static_cast<T*>(this)->template notifyOne<callerEnvironment>();
	}

	template <CallerEnvironment callerEnvironment>
	inline void notifyAll() {
		static_cast<T*>(this)->template notifyAll<callerEnvironment>();
	}

	template <CallerEnvironment callerEnvironment>
	void notifySpecific(workerid_t workerId) {
		// Do not notify ourselves
		if constexpr (callerEnvironment == CallerEnvironment::EMPER) {
			if (workerId == Worker::getCurrentWorkerId()) return;
		}

		static_cast<T*>(this)->template notifySpecific<callerEnvironment>(workerId);
	}

	void sleep() { static_cast<T*>(this)->sleep(); }

	inline void printStats(std::ostream& out = std::cout) { static_cast<T*>(this)->printStats(out); }
};
}	 // namespace emper::sleep_strategy
