// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2021 Florian Fischer
#include "sleep_strategy/Stats.hpp"

#include <memory>
#include <vector>

#include "WorkerLocalData.hpp"
#include "sleep_strategy/WorkerStats.hpp"

namespace emper::sleep_strategy {

thread_local WorkerStats* Stats::stats;

Stats::Stats(Runtime& runtime) {
	if constexpr (emper::STATS) {
		allStats = std::make_unique<WorkerLocalData<WorkerStats>>(
				[] {
					stats = new WorkerStats();
					return stats;
				},
				runtime);
	}
}

Stats::~Stats() {
	if constexpr (emper::STATS) {
		allStats->forEach([](auto* stats) { delete stats; });
	}
}

void Stats::print(std::ostream& out) {
	WorkerStats totalStats;
	for (auto& stats : allStats->getSnapshot()) {
		totalStats += stats;
	}

	totalStats += anywhereStats;

	out << totalStats;
}
}	 // namespace emper::sleep_strategy
