// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2021 Florian Schmaus
#include "AbstractWorkStealingWorkerStats.hpp"

#include <algorithm>

auto AbstractWorkStealingWorkerStats::operator+=(const AbstractWorkStealingWorkerStats& other)
		-> AbstractWorkStealingWorkerStats& {
	scheduledFibersToLocal += other.scheduledFibersToLocal;
	scheduledFibersToOverflowQueue += other.scheduledFibersToOverflowQueue;
	scheduledFibersToMpscQueue += other.scheduledFibersToMpscQueue;
	maxQueueLength = std::max(maxQueueLength, other.maxQueueLength);
	nextFiberFromLocal += other.nextFiberFromLocal;
	nextFiberFromHintLocal += other.nextFiberFromHintLocal;
	nextFiberFromHintAnywhere += other.nextFiberFromHintAnywhere;
	nextFiberStolen += other.nextFiberStolen;
	nextIoFiberStolen += other.nextIoFiberStolen;
	nextFiberFromAnywhereQueue += other.nextFiberFromAnywhereQueue;

	totalFibersLiftedFromAnywhereQueue += other.totalFibersLiftedFromAnywhereQueue;
	maxFibersLiftedFromAnywhereQueue =
			std::max(maxFibersLiftedFromAnywhereQueue, other.maxFibersLiftedFromAnywhereQueue);
	avgFibersLiftedFromAnywhereQueue.update(other.avgFibersLiftedFromAnywhereQueue.getAverage());

	return *this;
}

void AbstractWorkStealingWorkerStats::recordFibersLiftedFromAnywhereQueue(
		size_t fibersLiftedFromAnywhereQueue) {
	totalFibersLiftedFromAnywhereQueue += fibersLiftedFromAnywhereQueue;
	maxFibersLiftedFromAnywhereQueue =
			std::max(maxFibersLiftedFromAnywhereQueue, fibersLiftedFromAnywhereQueue);
	avgFibersLiftedFromAnywhereQueue.update(fibersLiftedFromAnywhereQueue);
}

void AbstractWorkStealingWorkerStats::recordScheduledToLocalAndQueueLength(uint64_t queueLength) {
	scheduledFibersToLocal++;
	maxQueueLength = std::max(maxQueueLength, queueLength);
}
