// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020-2021 Florian Fischer
#include <cerrno>		// for ETIME
#include <cstdint>	// for int32_t

#include "BinaryPrivateSemaphore.hpp"
#include "emper.hpp"
#include "fixtures/assert.hpp"
#include "io/Future.hpp"	// for AlarmFuture

using emper::io::AlarmFuture;
using emper::io::Future;

static void signallingCallbackTest() {
	AlarmFuture::Timespec ts = {.tv_sec = 1, .tv_nsec = 0};
	AlarmFuture alarm(ts);
	BPS bps;

	alarm.setCallback([&bps](int32_t res) {
		ASSERT(res == -ETIME);
		bps.signal();
	});
	alarm.submit();

	// wait till the callback was executed
	bps.wait();
}

static void spawningCallbackTest() {
	AlarmFuture::Timespec ts = {.tv_sec = 1, .tv_nsec = 0};
	AlarmFuture alarm(ts);
	BPS bps;

	alarm.setCallback([&bps](int32_t res) {
		ASSERT(res == -ETIME);
		async([bps = &bps] { bps->signal(); });
	});
	alarm.submit();

	// wait till the callback was executed
	bps.wait();
}

static void blockingCallbackTest() {
	AlarmFuture::Timespec ts = {.tv_sec = 1, .tv_nsec = 0};
	AlarmFuture alarm1(ts), alarm2(ts);
	BPS afterAlarm1, afterAlarm2, done;

	alarm1.setCallback(
			[afterAlarm1 = &afterAlarm1, afterAlarm2 = &afterAlarm2, done = &done](int32_t res) {
				ASSERT(res == -ETIME);
				afterAlarm1->signal();
				afterAlarm2->wait();
				done->signal();
			});
	alarm1.submit();

	afterAlarm1.wait();
	alarm2.submitAndWait();
	afterAlarm2.signal();

	// wait till the callback was executed
	done.wait();
}

void emperTest() {
	signallingCallbackTest();
	spawningCallbackTest();
	blockingCallbackTest();
}
