# SPDX-License-Identifier: LGPL-3.0-or-later
# Copyright 2021 Florian Fischer
import gdb

class WorkerFrameFilter():
    """Frame filter wrapping the frame iterator into an WorkerFrameIterator"""
    def __init__(self):
        self.name = "WorkerFrameFilter"
        self.priority = 100
        self.enabled = True
        gdb.frame_filters[self.name] = self

    def filter(self, frame_iter):
        return WorkerFrameIterator(frame_iter)


class WorkerFrameIterator:
    """Frame iterator skiping each frame without a valid name and function
    
    This is usefull in EMPER because worker stacks are allocated by emper and not glibc
    and are not properly walked by gdb resulting in multiple 'broken' frames above
    the dispatch loop.
    """
    def __init__(self, ii):
        self.input_iterator = ii

    def __iter__(self):
        return self

    def __next__(self):
        while True:
            frameDecorator = next(self.input_iterator)
            frame = frameDecorator.inferior_frame()

            if frame.name() or frame.function():
                return frameDecorator


if __name__ == '__main__':
    WorkerFrameFilter()
